package visualisation;
import header.WavFileHeader;
import header.WaveFormatEx;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.FileDialog;
import java.awt.Toolkit;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import javax.swing.JFrame;
import java.awt.Scrollbar;
import java.awt.event.AWTEventListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.KeyEvent;
/**
 * Abstrakter Erzeuger und Ansichtsklasse
 * @author Michael Steppat
 *
 */

public class Application extends JFrame implements AWTEventListener  {

	private static final long serialVersionUID = 1L;
	public AudioDocument audioDocument;
	View view;
	public RandomAccessFile file;
	Scrollbar scrollbar;
	Boolean isZooming = false;
	/**
	 * Konstruktor der Ansichtsklasse
	 */
	public Application () {		
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize(600, 330);		
		view = new View(this);	
		//addKeyListener(this);
		add(view);	
		scrollbar = new Scrollbar(Scrollbar.HORIZONTAL, 0, 200, 0, 2000);		
		
		Toolkit tk = Toolkit.getDefaultToolkit();
		tk.addAWTEventListener(this, AWTEvent.KEY_EVENT_MASK | AWTEvent.ADJUSTMENT_EVENT_MASK);
		add(BorderLayout.SOUTH, scrollbar);		
		setVisible(true);		
	}
	/**
	 * ffnet eine Wavedatei
	 * @return Gibt true zurck, wenn das ffnen erfolgreich war, ansonsten false (z.B. wenn kein Dateiname angegeben wurde)
	 */
	public Boolean openFile() {		
		FileDialog fileDialog = new FileDialog(this, "Datei ffnen", FileDialog.LOAD);
		fileDialog.setDirectory(".wav");
		fileDialog.setVisible(true);
		String fileName = fileDialog.getFile();
				
		if(fileName == null)
			return false;
		if(file != null) {
			try {
				file.close();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			file = null;
		}
		try {
			file = new RandomAccessFile(fileName, "r");
		} catch (FileNotFoundException e) {			
			e.printStackTrace();
		}		
		WavFileHeader wavHeader = new WavFileHeader();
		WaveFormatEx waveFormatEx = new WaveFormatEx();
		try {
			int fileLen = wavHeader.readHeader(file, waveFormatEx);
			System.out.println(fileLen);
			audioDocument.headerLength = (int)file.getFilePointer();
			audioDocument.fileLength = 10 * fileLen / 882;
			scrollbar.setMaximum(audioDocument.fileLength);
			System.out.println(audioDocument.fileLength);
			audioDocument.file = file;
			audioDocument.calculate();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return true;				
	}
	/**
	 * Implementierung des Interfaces AWTListener zur Verarbeitung von Benutzereingaben	
	 */
	
	public void eventDispatched(AWTEvent arg0) {
		if(arg0 instanceof KeyEvent) {
			//System.out.println("KeyEvent");
			KeyEvent ke = (KeyEvent)arg0;
			int key = ke.getKeyCode();
			if(ke.isShiftDown()) {
				isZooming = true;
				if(key == KeyEvent.VK_UP) {
					audioDocument.zoom(0, 1);
					view.repaint();				
				}
				if(key == KeyEvent.VK_DOWN) {
					audioDocument.zoom(0, 0);
					view.repaint();
				}
			}
			else
				isZooming = false;
			if(ke.isControlDown()) {
				isZooming = true;
				if(key == KeyEvent.VK_UP) {
					audioDocument.zoom(1, 1);
					view.repaint();
				}
				if(key == KeyEvent.VK_DOWN) {
					audioDocument.zoom(1, 0);
					view.repaint();
				}
			}
			else
				isZooming = false;
		}
		if(arg0 instanceof AdjustmentEvent) {
			//System.out.println("AdjustmentEvent");
			AdjustmentEvent ae = (AdjustmentEvent)arg0;
			if(ae.getSource() == scrollbar && isZooming == false) {
				audioDocument.SetFilePos(ae.getValue());
				audioDocument.calculate();
				view.repaint();
			}
		}
		
	}	
}
