package visualisation;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
/**
 * Konkreter Erzeuger mit Fabrikmethode zur Auswahl der Ansichten. Enthlt als Einstiegspunkt die Hauptfunktion.
 * @author Michael Steppat
 *
 */

public class AudioApplication extends Application implements ActionListener {
	/**
	 * Versionsbezeichner
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Referenz auf das Menelelement zur Auswahl der Zeitfunktion
	 */
	MenuItem mnuTimefunction;
	/**
	 * Referenz auf das Menelelement zur Auswahl des Betragsspektrums
	 */
	MenuItem mnuPowerSpectrum;
	/**
	 * Referenz auf das Menelelement zur Auswahl des Phasenspektrums
	 */
	MenuItem mnuPhaseSpectrum;
	/**
	 * Im Konstruktor wird das Men zur Auswahl der einzelnen Ansichten erzeugt
	 */
	
	public AudioApplication() {
		MenuBar mMenuBar = new MenuBar();
		setMenuBar(mMenuBar);
		Menu mMenu = new Menu("Datei");
		mMenuBar.add(mMenu);
		mnuTimefunction = new MenuItem("Zeitfunktion");
		mMenu.add(mnuTimefunction);
		mnuTimefunction.addActionListener(this);
		mnuPowerSpectrum = new MenuItem("Betragsspektrum");
		mMenu.add(mnuPowerSpectrum);
		mnuPowerSpectrum.addActionListener(this);
		mnuPhaseSpectrum = new MenuItem("Phasenspektrum");
		mMenu.add(mnuPhaseSpectrum);
		mnuPhaseSpectrum.addActionListener(this);		
	}
	/**
	 * Implementierung des Interfaces ActionListener. Je nach Menauswahl wird mit der Fabrikmethode createAudioDocument eine Ansicht der Zeitfunktion, des Betragspektrums oder des Phasenspektrums erzeugt.
	 */
	public void actionPerformed(ActionEvent event) {
		audioDocument = null;		
		if(event.getSource() == mnuTimefunction) {			
			audioDocument = createAudioDocument("Timefunction");
			if(openFile()== false)
				return;			 
			view.repaint();			
		}		
		if(event.getSource() == mnuPowerSpectrum) {
			audioDocument = createAudioDocument("PowerSpectrum");
			if(openFile()== false)
				return;
			view.repaint();			
		}
		if(event.getSource() == mnuPhaseSpectrum) {
			audioDocument = createAudioDocument("PhaseSpectrum");
			if(openFile()== false)
				return;
			view.repaint();			
		}		
	}
	/**
	 * Fabrikmethode zur Erzeugung der entsprechenden Ansicht. 
	 * @param choice Zur Auswahl der Ansicht wird ein String mit der entsprechenden englischen Bezeichnung bergeben. 
	 * @return Eine Referenz auf das konkrete Produkt vom Typ AudioDocument
	 */
	
	public AudioDocument createAudioDocument(String choice) {
		if(choice == "Timefunction")
			return new Timefunction();
		if(choice == "PowerSpectrum")
			return new PowerSpectrum();
		if(choice == "PhaseSpectrum")
			return new PhaseSpectrum();
		return null;
	}
	/**
	 * Hauptfunktion zum Starten der Anwendung
	 * @param args Keine Parameter erforderlich
	 */
	public static void main(String[] args) {
		new AudioApplication();
	}
	
}
