package visualisation;
import java.awt.Color;
import java.awt.Graphics;

import java.awt.Rectangle;
import java.io.IOException;
import java.text.DecimalFormat;
/**
* In dieser Klasse wird eine FFT der Audioabtastwerte durchgefhrt und die berechneten Werte in den Anzeigepuffer fr die Kurvendarstellung (curveValues) geladen. 
* Die Werte werden als Phasenspektrum ausgegeben
* @author Michael Steppat
*
*/
public class PhaseSpectrum extends AudioDocument{
	private static final long serialVersionUID = 1L;
	double f0 = 0, f1 = 20000;
	
	float xr[] = new float[1024];
	float xi[] = new float[1024];
	/**
	 * Im Konstruktor wird die Pufferlnge initialisiert
	 */
	public void PhaseSpektrum()
	{
		bufferLength = 1024 * 10 / 441;
	}
	/**
	 * Berechnung der Kurvenform
	 */
	public void calculate()  {
		FourierTransform fourierTransform = new FourierTransform();
		int i;
		short[] b = {0, 0};
		try {			
			file.seek(GetFilePos() * 882 / 10 + headerLength);
			for(i = 0; i < 1024; i++) {
				b[0] =  file.readByte();
				b[1] =  file.readByte();
				xr[i] =  (short)(0xff & b[0] | ((0xff & b[1]) << 8));
				xi[i] = 0.0f;				
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		fourierTransform.fft(xr, xi, 1024);
		double phase;
		curveSize = 500;
		for(i = 0; i < 500;i++) {
			phase = Math.atan2(xi[i], xr[i] );
			curveValues[2 * i] = (int)(phase * -100 / Math.PI + 100);
			curveValues[2 * i + 1] = (int)(phase * -100 / Math.PI + 100);
		}	
	}
	/**
	 * Zeichnen der Kurvenbeschriftung
	 * @param g Referenz auf ein Graphics-Objekt	 
	 */	
	
	public void drawCaptions(Graphics g) {	
		int x, y;
			
		// Beschriftung, y-Achse:		
		Color gridColor = new Color(0, 128, 64);		
		g.setColor(gridColor);
		String msg = new String();
		DecimalFormat df = new DecimalFormat( "'  '###;'-'###" );
				
		for(y = 0; y <= 10; y++) {				
			msg = df.format( (double)-y * 36 + 180 );			
			g.drawString(msg, 2, 20 * y + 14);
		}
		// x-Achse:
		double dFreqRange = f1 - f0;
		df.applyPattern("#####");
	
		for(x = 0; x <= 5; x++) {
			msg = df.format((double)x * dFreqRange / 5 + f0);
			g.drawString(msg, 1000 * x / 10 + 42, 226);			
		}
		// Angabe was angezeigt wird:		
		g.drawString("PHASE", 400, 246);
		
		// Zeitangabe:		
		msg = "T: " + df.format(GetFilePos());
		g.drawString(msg, 42, 246);		
				
	}
	/**
	 * /**
	 * Nicht implementiert
	 */
	 
	@Override
	void zoom(int coordinate, int direction) {
		// TODO Auto-generated method stub
		
	}
}
