package visualisation;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.IOException;
import java.text.DecimalFormat;
/**
 * Implementierung der abstrakten Methoden von AudioDocument zur Darstellung der Zeitfunktion
 * @author Michael Steppat
 *
 */
public class Timefunction extends AudioDocument{
	/**
	 * Versionsbezeichner
	 */
	private static final long serialVersionUID = 1L;
	/** 
	 * Horizontale Position des Zooms
	 */
	int horizontalZoomPos = 0;
	/**
	 * Array fr die Werte mit den Anzahlen, wie viele Abtastwerte im Fenster dargestellt werden
	 */
	int[] horizontalZoomLength = new int[10];
	int verticalZoomPos = 0;
	/**
	 * Array fr die Werte des angezeigten mV-Bereiches (1000 mV = Maximalaussteuerung)
	 */
	int[] verticalZoomSize = new int[10];
	/**
	 * Basis fr die Spannungsachse (mV pro Kstchen)
	 */
	double voltageBase = 200;
	/**
	 * Basis fr die Zeitachse (Samples pro Pixel)
	 */
	double timeBase = 1; 
	/**
	 * Im Konstruktor wird eine Tabelle fr die einzelnen Zoomstufen angelegt
	 */
	public Timefunction() {		
		horizontalZoomLength[0] = 441; 
		horizontalZoomLength[1] = 882; 
		horizontalZoomLength[2] = 2205; 
		horizontalZoomLength[3] = 4410; 
		horizontalZoomLength[4] = 8820; 
		horizontalZoomLength[5] = 22050; 
		horizontalZoomLength[6] = 44100; 
		horizontalZoomLength[7] = 88200;

		verticalZoomSize[0] = 1;
		verticalZoomSize[1] = 2;
		verticalZoomSize[2] = 4;
		verticalZoomSize[3] = 10;
		verticalZoomSize[4] = 20;
		verticalZoomSize[5] = 40;
		verticalZoomSize[6] = 100;
		verticalZoomSize[7] = 200;
		verticalZoomSize[8] = 500;
		verticalZoomSize[9] = 1000;
		bufferLength = horizontalZoomLength[horizontalZoomPos] * 10 / 441;
	}
	/**
	 * Berechnung der Kurvenform
	 */
	public void calculate() {
		int x = 0;
		int	pos;
		short	max = -32768;
		short	min =  32767;
		short[] b = {0, 0};
		short 	audioBuffer[] = new short[200000];
		
		// Werte lesen:
		int i;
		int bytePos;
		try {
			bytePos = GetFilePos() * 882 / 10;
			bytePos /= 2;
			bytePos *= 2;
			file.seek(bytePos + headerLength);
			for(i = 0; i < horizontalZoomLength[horizontalZoomPos]; i++) {
				b[0] =  file.readByte();
				b[1] =  file.readByte();
				audioBuffer[i] = (short)(0xff & b[0] | ((0xff & b[1]) << 8));								
			}
		}
		catch(IOException e) {
			e.printStackTrace();			
		}
		curveSize = 440;		
		for(i = 0; i < horizontalZoomLength[horizontalZoomPos]; i++) {
			// Maximalwert bestimmen:
			if(audioBuffer[i] > max)
				max = audioBuffer[i];
			// Minimalwert bestimmen:
			if(audioBuffer[i] < min)
				min = audioBuffer[i];

			// Position auf der Zeitachse:
			pos = (int)Math.round((double)i * 441.0 / (double)horizontalZoomLength[horizontalZoomPos]);
		
			if(pos > x) {
				curveValues[2 * x] = 100 - min * 100 * verticalZoomSize[verticalZoomPos] / 32768;
				curveValues[2 * x + 1] = 100 - max * 100 * verticalZoomSize[verticalZoomPos] / 32768;
					
				// Maximal- und Minimalwert zurcksetzen:
				max = -32768;
				min =  32767;
				// x inkrementieren:
				x++;
			}	
		}	
	}
	/**
	 * Zeichnen der Kurvenbeschriftung
	 * @param g Referenz auf ein Graphics-Objekt	 
	 */	
	public void drawCaptions(Graphics g) {	
		int x, y;
	
		// Beschriftung, y-Achse:		
		Color gridColor = new Color(0, 128, 64);		
		g.setColor(gridColor);
		String msg = new String();
		DecimalFormat df;
		if(voltageBase < 0.5)
			df = new DecimalFormat( "'  '####.#;'-'####.#" );
		else
			df = new DecimalFormat( "'  '####;'-'####" );
		for(y = 0; y <= 10; y++) {			
			msg = df.format((double)(5 - y) * voltageBase);			
			g.drawString(msg, 2, 20 * y + 14);
		}
		// x-Achse:		
		df.applyPattern("#####");
	
		for(x = 0; x <= 5; x++) {
			msg = df.format(x * 2 * timeBase + GetFilePos());
			g.drawString(msg, 1000 * x / 10 + 42, 226);			
		}
		// Angabe was angezeigt wird:		
		g.drawString("TIME", 400, 246);				
				
	}
	/**
	 * Horizontaler oder vertikaler Zoom der Ansicht
	 * @param coordinate Festlegung welche Achse gezoomt werden soll, 0 fr vertikalen 1 fr horizontalen Zoom
	 * @param direction Zoomrichtung, 0 fr herauszoomen, 1 zum hineinzoomen
	 */
	@Override
	void zoom(int coordinate, int direction) {
		if(coordinate == 0) {
			if(direction == 0)
				verticalZoomPos--;
			if(direction == 1)
				verticalZoomPos++;
		}
		if(verticalZoomPos < 0)
			verticalZoomPos = 0;
		if(verticalZoomPos > 9)
			verticalZoomPos = 9;
		voltageBase = 200.0 / (double)verticalZoomSize[verticalZoomPos];
		
		if(coordinate == 1) {
			if(direction == 0)
				horizontalZoomPos--;
			if(direction == 1)
				horizontalZoomPos++;
		}
		if(horizontalZoomPos < 0)
			horizontalZoomPos = 0;
		if(horizontalZoomPos > 7)
			horizontalZoomPos = 7;	
		timeBase = (double)horizontalZoomLength[horizontalZoomPos] / 441.0;
		bufferLength = horizontalZoomLength[horizontalZoomPos] * 10 / 441;
		calculate();		
	}
}
