package header;

import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * Hilfsklasse fr das Schreiben des AIFF-Dateiheaders 
 * @author Michael Steppat
 *
 */

public class AiffFileHeader {	
	/**
	 * Umwandlung von einem 64-Bit double Wert in einen 80-Bit double Wert. Dieser wird zum Speichern der Samplerate bentigt.
	 * 
	 * @param outputValue 	Ausgabewert als Byte-Array mit 10 Elementen
	 * @param inputValue 	Eingabewert
	 */
	private void DoubleToLongDouble(byte[] outputValue, double inputValue) {
		//Berechnung des Exponenten:
		int exponent = (int)Math.floor(Math.log(inputValue) / Math.log(2));
		// Berechnung der Mantisse:
		double temp = Math.pow(2, 63) * (inputValue - Math.pow(2, exponent)) / Math.pow(2, exponent);
		long fraction = (long)temp;
		long ipart = 0x80000000;
		ipart = ipart << 32;
		fraction = fraction | ipart;
		// Werte in das Datenfeld eintragen:
		int biasedExponent = exponent + 16383;
		outputValue[0] = (byte)(biasedExponent >> 8);
		outputValue[1] = (byte)biasedExponent;
		outputValue[2] = (byte)(fraction >> 56);
		outputValue[3] = (byte)(fraction >> 48);
		outputValue[4] = (byte)(fraction >> 40);
		outputValue[5] = (byte)(fraction >> 32);
		outputValue[6] = (byte)(fraction >> 24);
		outputValue[7] = (byte)(fraction >> 16);
		outputValue[8] = (byte)(fraction >> 8);
		outputValue[9] = (byte)(fraction >> 0);		
	}
	/**
	 * 
	 * @param file 				Referenz auf das RandomAccesFile-Objekt (Dateizugriffsfunktionen)
	 * @param waveFormatEx		Referenz auf die Formatdaten
	 * @param numSampleFrames	Anzahl der Samples 
	 * @throws IOException		Wird bei Zugriffsproblemen geworfen
	 */

	public void writeHeader(RandomAccessFile file, WaveFormatEx waveFormatEx, int numSampleFrames) throws IOException{
		file.seek(0);
		byte[] chunk0 = {'F', 'O', 'R', 'M'};
		file.write(chunk0);
		int len = (int)file.length() - 8;
		file.writeInt(len);		
		
		byte[] chunk1 = {'A', 'I', 'F', 'F'};
		file.write(chunk1);
		byte[] chunk2 = {'C', 'O', 'M', 'M'};
		file.write(chunk2);
		len = 18;			// Lnge der auf den COMM -Chunk folgenden Daten
		file.writeInt(len);
		file.writeShort(waveFormatEx.nChannels);
		file.writeInt(numSampleFrames);	
		file.writeShort(waveFormatEx.wBitsPerSample);
		byte[] sampleRate;
		sampleRate = new byte[10];
		DoubleToLongDouble(sampleRate, waveFormatEx.nSamplesPerSec);
		file.write(sampleRate);
		
		byte[] chunk3 = {'S', 'S', 'N', 'D'};
		file.write(chunk3);
		len = (int)file.length() - 54;
		file.writeInt(len);
		int offset = 0;
		file.writeInt(offset);
		int blockSize = 0;
		file.writeInt(blockSize);
	}
}
