package modeling;
/**
 * Antialiasing-Filter zur Bandbegrenzung bei Oversampling
 * @author Michael Steppat
 *
 */
public class AntiAlias {
	double[] a;
	double[] buffer;
	int pos;
	int N;
	/**
	 * Konstruktor
	 * @param omega Relative Eckfrequenz (Kehrwert des Oversampling-Faktors)
	 * @param filterLength Lnge der Impulsantwort
	 */
	AntiAlias(double omega, int filterLength) {
		int i;
		pos = 0;
		filterLength *= 2;
		filterLength /= 2;
		N = filterLength;

		buffer = new double[N];
		a = new double[N];
		a[N / 2] = a[N / 2 - 1] = omega;

		for(i = 1; i < (N / 2); i++) {
			a[N / 2 + i] = a[N / 2 - 1 - i] = Math.sin(omega * (double)i * Math.PI) / (Math.PI * i);	
		}
	}
	/**
	 * Fhrt die Faltung (Convolution) durch
	 * @param val bergebener Abtastwert der zu bearbeitenden Abtastfolge
	 * @return Mit Bercksichtigung der Faltung neu berechneter Abtastwert
	 */
	double Convolve(double val) {
		double ret;
		int i;
		int addr;
		buffer[pos] = val;
		ret = 0;
		
		for(i = 0; i < N; i++) {
			addr =  pos - i;
			if(addr < 0)
				addr += N;
			ret += a[i] * buffer[addr];		
		}
		pos++;
		if(pos > N - 1)
			pos = 0;
		return ret;
	}
}
