package modeling;

import java.io.RandomAccessFile;
import org.json.JSONObject;
/**
 * Implementierung der abstrakten Methoden der Klasse AcousticObject zur Generierung eines Masse-Feder-Dmpfersystems
 * @author Michael Steppat
 *
 */
public class MSDSystem extends AcousticObject {	
	// Daten des schwingenden Systems:
	double m;		// Masse
	double s;		// Federkonstante
	double r;		// Dmpfung
	// Anregekraft:
	double Fa = 0;
	double Fmax = 100;
	int N = 22;
	/**
	 * Erzeugt mit Hilfe der Newmark-Integration die Schwingung eines Masse-Feder-Dmpfersystems
	 */
	public void createWave() {
		// Bewegungsgren:
		double x = 0;	// Auslenkung
		double v = 0;	// Geschwindigkeit
		double a = 0;	// Beschleunigung
		// innere Krfte
		double Fm = 0;	// Massentrgheitskraft
		double Fd = 0;	// Dmpfungskraft
		double Fs = 0;	// Federrckstellkraft		
		double dt = 1.0 / 44100.0;
		double out[] = new double[audioDataLength];
		double max = 0.0;
		int i;
		
		for(i = 0; i < audioDataLength; i++) {
			// Berechnung der ueren Kraft:
			Fa = 0;
			if(i <= N / 2)
					Fa = Fmax * (double)i / (N / 2);
			if(i > N / 2 && i <= N)
					Fa = Fmax * (double)(N - i)  / (N / 2);
			// Berechnung der inneren Krfte:
			// Federkraft
			Fs = s * x;			
			// Dmpfung:
			Fd = r * v;
			// Krftegleichung:		
			Fm = Fa - Fd - Fs; 
			// Integration:
			a = Fm / m;
			v = v + dt * a;
			x = x + dt * v;
			out[i] = v;			
			if(Math.abs(v) > max)
				max = Math.abs(v);
		}
		for(i = 0; i < audioDataLength; i++) {
			audioData[i] = (short)(32600.0 * out[i] / max);
		}
	}
	/**
	* Liest die JSON-Datei mit den Parametern
	* @param fileName Name der JSON-Datei (Konfigurationsdatei)
	*/
	public void ReadJSON(String fileName) {
		try {
			java.lang.String the_json;
			RandomAccessFile file;
			System.out.println(fileName);
			file = new RandomAccessFile(fileName, "r");
			String line;
			the_json = "";
			while((line = file.readLine()) != null)
				the_json += line;			
			JSONObject myjson = new JSONObject(the_json); 
			audioFileName = myjson.getString("Ausgabedatei");
			System.out.println(audioFileName);
			m = myjson.getDouble("Masse");
			System.out.println("m = "+ m);
			s = myjson.getDouble("Federkonstante");
			System.out.println("s = "+ s);
			r = myjson.getDouble("Daempfung");
			System.out.println("r = "+ r);			 
 
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}				
	}
	/**
	 * Hauptfunktion
	 * @param args Name der Konfigurationsdatei
	 */
	public static void main(String[] args) {
		MSDSystem msdSystem = new MSDSystem();
		msdSystem.doModelling(args[0]);
	}
}