package modeling;

import java.io.RandomAccessFile;

import org.json.JSONObject;
/**
 * Implementierung der abstrakten Methoden der Klasse AcousticObject zur Generierung des Klangs einer Platte
 * @author Michael Steppat
 *
 */
public class Plate extends AcousticObject {	
	double plateLength;
	double plateWidth;
	double plateHeight;
	double xPos, yPos;
	double E;
	double ny;
	double rho;
	double eta;
	double amp;
	double dl;
	int oversampling;
	/**
	 * Erzeugt mit Hilfe des Finite-Differenzen-Verfahrens die Schwingung einer Platte
	 */	
	public void createWave() {
		double w[][][] = new double[3][64][64];
		int j, k;
		double Fa, Fmax = 1000 * oversampling;
		int N = (int)(44.1 * 0.1);
		double dt = 1.0 / (44100.0 * (double)oversampling);
		double dx = dl;
		double dy = dx;
		int xnodes = (int)(plateWidth / dx);
		int ynodes = (int)(plateLength / dy);
		double h = plateHeight;
		double K = E * Math.pow(h, 3) / (12 * (1 - Math.pow(ny, 2)));
		double c1 = -Math.pow(dt, 2) * K / (Math.pow(dx, 4) * rho * h);
		double c2 = -Math.pow(dt, 2) * K / (Math.pow(dy, 4) * rho * h);
		double c3 = -Math.pow(dt, 2) * K * 2 / (dx * dx *  dy * dy * rho * h);
		double c4 = eta * dt / (rho * h * 2);
		double c5 = -Math.pow(dt, 2) / (rho * h);		
		double b = 1 / (1  +  c4);				
		int n;
		int m;
		double out;
		out = 0;
		m = 0;
		n = 0;
		double nyq = 1.0 / (double)oversampling;
		AntiAlias filter1 = new AntiAlias(nyq, 40);
		AntiAlias filter2 = new AntiAlias(nyq, 40);
		AntiAlias filter3 = new AntiAlias(nyq, 40);
		int xpos = (int)((double)xnodes * xPos )  + 2;
		int ypos = (int)((double)ynodes * yPos )  + 2;
		if(xpos < 1)
			xpos = 1;
		if(ypos < 1)
			ypos = 1;
		for(int i = 0; i < 88200 * oversampling; i++) {
			// Berechnung der ueren Kraft:
			Fa = 0;
			if(i <= N / 2)
				Fa = Fmax * (double)i / (N / 2);
			if(i > N / 2 && i <= N)
				Fa = Fmax * (double)(N - i)  / (N / 2);		
			// Auslenkung berechnen
			for(k = 3; k < ynodes + 2 ; k++)			// y-Achse	
				for(j = 3; j < ynodes + 2; j++) {		// x-Achse				
					// Differenzengleichung der Platte:
					w[2][j][k] = b * (c1*(w[1][j+2][k]-8*w[1][j+1][k]+20*w[1][j][k]-8*w[1][j-1][k]+w[1][j-2][k])+ // Sternoperator
								 c2*(w[1][j][k+2]-8*w[1][j][k+1]/*+20*w[1][j][k]*/-8*w[1][j][k-1]+w[1][j][k-2])+  //
								 c3*(w[1][j-1][k+1] + w[1][j-1][k-1] + w[1][j+1][k+1] + w[1][j+1][k-1]) +		  // 13-teilig	
								 2*w[1][j][k]-w[0][j][k] + c4* w[0][j][k]);
					// Anregekraft:
					if(j == xpos  && k == ypos)
						w[2][j][k] += Fa * c5 * b;
				}
			
	        // Randwerte w:
			for(k = 2; k <= ynodes + 2; k++) {
				w[2][2][k] = 0;	
				w[2][1][k] = -w[2][3][k];
				w[2][xnodes + 2][k] = 0;	
				w[2][xnodes + 3][k] = -w[2][xnodes + 1][k];
				
			}

			for(j = 2; j <= xnodes + 2; j++) {
				w[2][j][2] = 0;	
				w[2][j][1] = -w[2][j][3];	
				w[2][j][ynodes + 2] = 0;	
				w[2][j][ynodes + 3] = -w[2][j][ynodes + 1];	
			
			}
			out = filter1.Convolve(32767.0 * amp * w[2][xpos][ypos]);
			out = filter2.Convolve(out);
			out = filter3.Convolve(out);	
			m++;
			if(m >= oversampling) {
				audioData[n] = (short)out;
				n++;
				m = 0;
			}
			if(out > 65536.) {				
				System.out.println("FDM calculation unstable");
				break;
			}
			for(k = 0; k <= ynodes + 4; k++) 
				for(j = 0; j <= xnodes + 4; j++) {
					w[0][j][k] = w[1][j][k];
					w[1][j][k] = w[2][j][k];			
				}			
		}		
	}
	/**
	* Liest die JSON-Datei mit den Parametern
	* @param fileName Name der JSON-Datei (Konfigurationsdatei)
	*/
	public void ReadJSON(String fileName) {
		try {
			java.lang.String the_json;
			RandomAccessFile file;
			System.out.println(fileName);
			file = new RandomAccessFile(fileName, "r");
			String line;
			the_json = "";
			while((line = file.readLine()) != null)
				the_json += line;			
			JSONObject myjson = new JSONObject(the_json); 
			audioFileName = myjson.getString("Ausgabedatei");
			System.out.println(audioFileName);
			plateLength = myjson.getDouble("Laenge");			
			plateWidth = myjson.getDouble("Breite");
			plateHeight = myjson.getDouble("Hoehe");
			xPos = myjson.getDouble("Anschlagposition(X)");
			yPos = myjson.getDouble("Anschlagposition(Y)");
			E = myjson.getDouble("Elastizitaetsmodul");
			ny = myjson.getDouble("Poissonzahl");
			rho = myjson.getDouble("Dichte");			
			eta = myjson.getDouble("Verlustfaktor");
			amp = myjson.getDouble("Verstaerkung");			
			dl = myjson.getDouble("Elementbreite");
			oversampling = myjson.getInt("Oversampling");
 
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}				
	}
	/**
	 * Hauptfunktion
	 * @param args Name der Konfigurationsdatei
	 */
	public static void main(String[] args) {
		Plate plate = new Plate();
		plate.doModelling(args[0]);
	}

}
