package astromech;


import header.WavFileHeader;
import header.WaveFormatEx;

import java.io.IOException;
import java.io.RandomAccessFile;
/**
 * Mit der Klasse R2D2_02 lsst sich eine kurze Tonfolge erzeugen   
 * @author Michael Steppat
 *
 */
public class R2D2_02 {
	/**
	 * Referenz auf das RandomAccessFile-Objekt
	 */
	private RandomAccessFile file;
	/**
	 * Referenz auf das WaveFormatEx-Objekt. Es enthlt die im Header gespeicherten Formatdaten
	 */
	private WaveFormatEx waveFormatEx;
	/**
	 * Erzeugt die Ausgabedatei und fllt das waveFormatEx mit den Formatdaten fr das Format PCM, 44100 Hz, 16-Bit, mono
	 * @param fileName
	 * @throws IOException
	 */
	public void createFile(String fileName) throws IOException {
		
		file = new RandomAccessFile(fileName, "rw");
		file.setLength(0);
		int i;		
		for(i = 0; i < 44; i++)
			file.writeByte(0);
		waveFormatEx = new WaveFormatEx();
		waveFormatEx.wFormatTag = 1;
		waveFormatEx.nChannels = 1;
		waveFormatEx.nSamplesPerSec = 44100;
		waveFormatEx.nAvgBytesPerSec = 88200;
		waveFormatEx.nBlockAlign = 2;
		waveFormatEx.wBitsPerSample = 16;
		waveFormatEx.cbSize = 0;			
	}
	/**
	 * Erzeugt das Audiosignal
	 * @throws IOException Wird bei Zugriffsproblemen geworfen.
	 */	
	public void createAudioData(double amplitude, double frequency, double phase, int length) throws IOException {		
		int i;
		phase = phase * Math.PI / 180;
		length = (int)(length * 44.1);
		short sample;
		for(i = 0; i < length; i++) {
			sample = (short)(amplitude * Math.sin(phase + 2 * Math.PI * frequency * i / 44100));
			sample += (short)(amplitude * Math.sin(phase + 2 * Math.PI * 3 * frequency * i / 44100));
			file.writeByte(sample);
			file.writeByte(sample >> 8);
		}
	}
	/**
	 * Erzeugt ein wavFileHeader-Objekt und schreibt die Formatdaten in die Datei
	 */
	public void writeHeader() throws IOException
	{
		WavFileHeader wavFileHeader = new WavFileHeader();
		wavFileHeader.writeHeader(file, waveFormatEx);		
	}
	/**
	 * Hauptfunktion	
	 * @param args Dateiname
	 */	
	public static void main(String[] args) {		
		
		R2D2_02 sinus = new R2D2_02();			
		try {		
			sinus.createFile(args[0]);
			sinus.createAudioData(16000, 2489, -90, 230);
			sinus.createAudioData(16000, 2217.5, -90, 45);
			sinus.createAudioData(16000, 2093, -90, 45);
			sinus.createAudioData(16000, 2217.5, -90, 45);
			sinus.createAudioData(16000, 2793.8, -90, 100);
			sinus.createAudioData(16000, 2489, -90, 100);
			sinus.createAudioData(16000, 2093, -90, 55);
			sinus.createAudioData(16000, 1975.5, -90, 55);
			sinus.createAudioData(16000, 2637.5, -90, 55);
			sinus.createAudioData(16000, 3520, -90, 55);
			sinus.createAudioData(16000, 1864.7, -90, 55);
			sinus.createAudioData(16000, 1760, -90, 110);			
			sinus.writeHeader();			
		} catch (IOException e) {			
			e.printStackTrace();
		}
		System.out.println("Wavedatei erzeugt");
	}
}
